﻿// Ctrader 3.5 and above.
// (c) Gonzalo Sanchez Pla - 2019 all rights reserved.
// This software can be used, modified and distributed by any means freely.
// This software and its derivatives cannot be sold nor incorporated to any commercial product.
// By selling this software without written permission, the seller accepts to pay $1000 per sold copy to the author mentioned on the comment above.

using System;
using cAlgo.API;
using cAlgo.API.Internals;
using cAlgo.API.Indicators;
using cAlgo.Indicators;
using System.Linq;


namespace cAlgo
{


    [Indicator(IsOverlay = true, TimeZone = TimeZones.UTC, AccessRights = AccessRights.None)]
    public class DailyCandleStrength : Indicator
    {

        private double daily_candle_open_price;
        private DateTime daily_candle_open_date;
        private MarketSeries H1;
        private Color main_label_color;

        public enum ReferenceTimeEnum
        {
            UTC = 0,
            NY = 12,
            UK = 7,
            GE = 6,
            AU = 21,
            JP = 23
        }

        public enum ColorosEnum
        {
            White = 0,
            Black = 1
        }


        [Parameter("Smoothing periods", DefaultValue = 89)]
        public int rolling_periods { get; set; }

        [Parameter("Open time reference", DefaultValue = ReferenceTimeEnum.UTC)]
        public ReferenceTimeEnum SelectedMarket { get; set; }

        [Parameter("Label color", DefaultValue = ColorosEnum.Black)]
        public ColorosEnum mainLabelColorEnum { get; set; }



        protected override void Initialize()
        {


            if (mainLabelColorEnum == ColorosEnum.Black)
            {
                main_label_color = Color.Black;
            }
            else if (mainLabelColorEnum == ColorosEnum.White)
            {
                main_label_color = Color.White;
            }

            DateTime server_time = Server.Time;
            DateTime utc_time = Server.TimeInUtc;
            int utc_hour = utc_time.Hour;

            DateTime server_target_date = server_time.AddHours(-1 * utc_hour + (int)SelectedMarket);
            DateTime server_target_datetime = new DateTime(server_target_date.Year, server_target_date.Month, server_target_date.Day, server_target_date.Hour, 0, 0);


            H1 = MarketData.GetSeries(Symbol.Name, TimeFrame.Hour);
            int open_time_index = H1.OpenTime.GetIndexByTime(server_target_datetime);
            daily_candle_open_price = H1.Open[open_time_index];
            daily_candle_open_date = H1.OpenTime[open_time_index];

        }


        public override void Calculate(int index)
        {

            string current_symbol = Symbol.Name;

            // SYMBOL VALUE //
            double symbol_pip_value = Symbol.PipValue;
            double symbol_pip_size = Symbol.PipSize;
            long symbol_lot_size = Symbol.LotSize;

            int N = rolling_periods;
            int number_of_bars = MarketSeries.High.Count;
            long slice_size = Math.Min(N, number_of_bars);

            int i = 0;
            double[] high_minus_low_segment = new double[slice_size];

            while (i < slice_size)
            {

                high_minus_low_segment[i] = MarketSeries.High[index - i] - MarketSeries.Low[index - i];
                i++;
            }


            double high_minus_low_median = Statistics.GetMedian(high_minus_low_segment);
            double high_minus_low_std = Statistics.StdDev(high_minus_low_segment);

            double high_minus_low_value_pips = (high_minus_low_median / symbol_pip_size) * symbol_pip_value;
            double high_minus_low_lot_value = high_minus_low_value_pips * symbol_lot_size;

            double high_minus_low_std_value_pips = (high_minus_low_std / symbol_pip_size) * symbol_pip_value;
            double high_minus_low_std_lot_value = high_minus_low_std_value_pips * symbol_lot_size;

            string high_minus_low_lot_value_str = string.Format("{0:0.00}", high_minus_low_lot_value);
            string high_minus_low_lot_value_lower_str = string.Format("{0:0.00}", high_minus_low_lot_value - high_minus_low_std_lot_value);
            string high_minus_low_lot_value_upper_str = string.Format("{0:0.00}", high_minus_low_lot_value + high_minus_low_std_lot_value);

            string lot_trade_value_label = "Active TF Lot Trade Value:\t" + high_minus_low_lot_value_str;
            lot_trade_value_label = lot_trade_value_label + "\t[lower: " + high_minus_low_lot_value_lower_str + ", upper: " + high_minus_low_lot_value_upper_str + "]";
            // END OF SYMBOL VALUE //


            // SYMBOL STRENGTH //

            double symbol_strength = 100 * (MarketSeries.Close[index] - daily_candle_open_price) / daily_candle_open_price;
            string symbol_strength_label = "Symbol Daily Strength: \t" + string.Format("{0:0.00}", symbol_strength) + "% \t(from " + SelectedMarket.ToString() + " opening)";
            // END OF SYMBOL STRENGTH //

            //Layout
            Chart.DrawStaticText("symbol_strength", symbol_strength_label, VerticalAlignment.Top, HorizontalAlignment.Left, main_label_color);
            Chart.DrawStaticText("trade_value", "\n" + lot_trade_value_label, VerticalAlignment.Top, HorizontalAlignment.Left, main_label_color);




        }
    }
}
